#! /usr/bin/env python
# -*- coding: utf-8 -*- 100 -*-
# @creator (C) 2003 Guido U. Draheim
# @license http://creativecommons.org/licenses/by-nc-sa/2.0/de/

from datetime import datetime
from match import Match
import os.path

class DocusFile:
    def __init__(self, filename):
        self.html = None
        self.scan(filename)
    def pagetype(self):
        return "docus"
    def html_page(self):
        return self.html
    def scan(self, filename):
        self.orig = filename
        self.name = filename[:-3] # .htm
        self.text = file(filename).read()
    def parse(self):
        x = Match()
        if self.text & x(r"<!--\s*CATEGORY\s*\"([^\"]+)\""):
            self.category = x[1]
            self.sort = x[1]
        if self.text & x(r"<!--\s*SORTING\s*\"([^\"]+)\""):
            self.sort = x[1]
        if self.text & x(r"<!--\s*CHAPTER\s*\"([^\"]+)\""):
            self.chapter = x[1]
        if self.text & x(r"<!--\s*SHORTNAME\s*\"([^\"]+)\""):
            self.title = x[1]

class HtmlDocusPage:
    def __init__(self, page, options = None, style = None):
        self.page = page
        self.o = options
        last_release = self.o.release.strip()
        if not last_release:
            last_release = self.o.version.strip()
        self.style = style
        self.title = None
        self.old_package_tar = self.o.package+".tar.bz2"
        self.old_macros_tar = self.o.package+"-macros.tar.bz2"
        self.old_man7_tar = self.o.package+"-man7.tar.bz2"
        self.new_package_tar = self.o.package+"-"+last_release+".tar.bz2"
        self.new_macros_tar = self.o.package+"-macros-"+self.o.version+".tar.bz2"
        self.new_man7_tar = self.o.package+"-man7-"+self.o.version+".tar.bz2"
    def pagetype(self):
        return self.page.pagetype()
    def get_name():
        return self.page.name
    def get_title(self):
        if not self._gen_html(): return None
        return self.title
    def html_info(self):
        if not self._gen_html(): return None
        return self.htmlpageinfo
    def html_text(self):
        if not self._gen_html(): return None
        return self.htmlpage
    def _gen_html(self):
        if not self.title:
            self.gen_html()
        return True
    def gen_html(self):
        filename = os.path.basename(self.page.orig)
        self.title = "Documentation - "+filename
        self.htmlpageinfo = "#"
        self.htmlpage = (self.expand(self.page.text)
                         .replace(self.old_package_tar, self.new_package_tar)
                         .replace(self.old_macros_tar, self.new_macros_tar)
                         .replace(self.old_man7_tar, self.new_man7_tar))
    def expand(self, text):
        if self.style:
            return self.style.expand(text)
        else:
            return text
        
