#! /usr/bin/env python
# -*- coding: utf-8 -*-
# @creator (C) 2003 Guido U. Draheim
# @license http://creativecommons.org/licenses/by-nc-sa/2.0/de/

from warn_ import *
import sys
import os
import os.path

n  = "\n"
br = "<br />\n"

class HtmlPage:
    equiv = { "content-type" : "text/html; charset=UTF-8" }
    def __init__(self, page, style):
        self.page = page
        self.style = style
        self.debugging = None
    def gen_html(self, navbar):
        page = self.page
        html_text = page.html_text()
        html_title = page.get_title()
        site_title = "Autoconf Macro Archive - "
        css_text = ""
        if self.style:
            if navbar: self.style.set_seen("a")
            html_text = self.style.expand(html_text)
            css_text = self.style.css()
        self.html = "" # doctype?
        self.html += "<html><head><title>"+site_title+html_title+"</title>"+n
        for name in self.equiv.keys():
            value = self.equiv[name]
            self.html += '<meta http-equiv="'+name+'" content="'+value+'" />'
        if css_text: self.html += "<style>"+n+css_text+n+"</style>"+n
        self.html += "</head><body>"+n
        self.html += ('<table width="100%"><tr valign="top">'
                      '  <td width="145" bgcolor="#E0E0FF">')
        if navbar:
            if "name" not in page.__dict__.keys() or "/" in page.name:
                self.html += navbar.html("../") ### <--
            else:
                self.html += navbar.html("") ### <--
        self.html += """</td><td>"""+n
        self.html += html_text ### <--
        self.html += "</td></tr></table>"+n
        if self.debugging:
            for x in page.__dict__.keys():
                self.html += " <small>"+x+"</small>"
        self.html += "</body></html>"+n    

class HtmlGenerator:
    def __init__(self, o, navbar = None, style = None):
        self.o = o
        self.navbar = navbar
        self.style = style
        self.builddir = "www"
        if o.output: self.builddir = o.output
        self.htm = ".html"
        if o.htm: self.htm = o.htm
    def with_navbar(self, navbar):
        self.navbar = navbar
        return self
    def with_style(self, style):
        self.style = style
        return self
    def www_copy_of_m4(self, page):
        if len(self.builddir) > 1 and page.pagetype() == "macro":
            self.create_www_copy_of_m4(page)
    def create_www_copy_of_m4(self, page):
        """ copy the macro m4 to the www target dir """
        filename = os.path.join(self.builddir, page.src.orig)
        dirname = os.path.dirname(filename)
        if not os.path.isdir(dirname) and "/" in dirname:
            os.makedirs(dirname)
        try:
            fd = file(filename,"w")
            fd.write(page.src.text)
            fd.close()
        except IOError, e:
            warn(filename+": could not open for writing", e)
    def generate_output(self, out): # main entry point
        print "generate_output()"
        testfile = self.o.test
        if not testfile or len(testfile) < 2: testfile = "test.out"
        if self.o.test: os.remove(testfile)
        for name in sorted(out.files.keys()):
            page = out.files[name]
            print >> sys.stderr, page.pagetype(),name,"..",
            self.www_copy_of_m4(page)
            fd = None
            if self.o.test:
                fd = self.open_testfile(testfile)
            else:
                htmlfilename = os.path.join(self.builddir, name + self.htm)
                fd = self.open_htmlfile(htmlfilename)
            if not fd: continue
            self.save_html(fd, page)
    def open_testfile(self, filename):
        try:
            return file(filename,"a")
        except IOError, e:
            warn(filename+": could not open for writing", e)
            return None        
    def open_htmlfile(self, filename):
        dirname = os.path.dirname(filename)
        if not os.path.isdir(dirname) and "/" in dirname:
            os.makedirs(dirname)
        try:
            return file(filename,"w")
        except IOError, e:
            warn(filename+": could not open for writing", e)
            return None        
    def save_html(self, fd, page):
        """ with fd already open """
        html = ""
        ee = None
        try:
            html = page.src.html
        except Exception, e:
            ee = e
            pass
        try:
            make = HtmlPage(page.htmlpage, self.style)
            make.debugging = self.o.debug
            make.gen_html(self.navbar)
            html = make.html
        except IOError, e:
            print ee, e
            pass
        if not html:
            fd.close()
            print >> sys.stderr, ".. SKIP"
        else:
            fd.write(html)
            fd.close()
            print >> sys.stderr, len(html)
    def gettext(self, out):
        doctext = ""
        for name in sorted(out.files.keys()):
            page = out.files[name]
            doctext += self.gettext_page(page)
        return doctext
    def gettext_page(self, page):
        html = ""
        try:
            html = page.src.html
        except Exception, e:
            ee = e
            pass
        try:
            make = HtmlPage(page.htmlpage, self.style)
            make.debugging = self.o.debug
            make.gen_html(self.navbar)
            html = make.html
        except IOError, e:
            text = "ERROR: (pagename?)" + "\n" + str(ee) + "\n" + str(e)
            raise Exception(text)
        return html
    
        

