#! /usr/bin/env python
# -*- coding: utf-8 -*-
# @creator (C) 2003 Guido U. Draheim
# @license http://creativecommons.org/licenses/by-nc-sa/2.0/de/

from warn_ import warn
import os.path
from datetime import date

def is_upper(x):
    return 'A' <= x and x <= 'Z'
def is_lower(x):
    return 'a' <= x and x <= 'z'
n  = "\n"
br = "<br />\n"
htm = ".html"

class IndexPage:
    def __init__(self, filename):
        self.html = None
        self.title = ""
        self.name = filename
    def pagetype(self):
        return "index"
    def html_page(self):
        return self.html
    def get_title(self):
        return self.title
    def get_name(self):
        return self.name
    def html_text(self):
        return self.htmlpage
    def html_info(self):
        return "info"
class HtmlIndexPage(IndexPage):
    def __init__(self, filename):
        IndexPage.__init__(self, filename)

class IndexCategory:
    _category_mapping = {
        "AM Support" : "AM Support",
        "C++ Support" : "C++ Support",
        "C Support" : "C Support",
        "Java Support" : "Java Support",
        "Fortran Support" : "Fortran Support",
        "LaTeX Support" : "LaTeX Support",
        "Cross Compilation" : "Cross Compilation",
        "Installed Packages" : "Installed Packages",
        "Miscellaneous" : "Miscellaneous",
        "System Headers" : "System Headers",
        "System Utilities" : "System Utilities",
        "Obsoleted" : "Obsoleted",
        "Automake" : "AM Support",
        "AM" : "AM Support",
        "Cxx" : "C++ Support",
        "C++" : "C++ Support",
        "C" : "C Support",
        "Java" : "Java Support",
        "Fortran" : "Fortran Support",
        "LaTeX" : "LaTeX Support",
        "CrossCompilation" : "Cross Compilation",
        "InstalledPackages" : "Installed Packages",
        "Misc" : "Miscellaneous",
        "Obsolete" : "Obsoleted" }
    def category_mapping(self, category_name):
        name = category_name.replace("_"," ")
        if name in self._category_mapping:
            return self._category_mapping[name]
        if name and is_upper(name[0]):
            warn("'"+name+"' is not a known main category")
        return name
    def __init__(self, page):
        self.page = page
        if page: self.parse(page)
    def parse(self, page):
        """ we have three category types - a topic category, a source
            category, and a status category"""
        pagedirectory = os.path.basename(os.path.dirname(page.name))
        if pagedirectory == page.name: pagedirectory = ""
        self.sourcedir_category = self.category_mapping(pagedirectory)
        self.explicit_category = self.category_mapping(page.category)
        # topic_category
        if self.explicit_category and is_upper(self.explicit_category[0]):
            self.topic_category = self.explicit_category
        else:
            self.topic_category = "Uncategorized"
        # source_category
        if self.sourcedir_category:
            self.source_category = self.sourcedir_category
        else:
            self.source_category = "unknown"
        # status_category
        if self.topic_category == "Obsoleted":
            self.status_category = "obsoleted"
        elif self.topic_category != "Obsoleted":
            self.status_category = "released"
        else:
            self.status_category = "experimental"

class IndexGenerator:
    def __init__(self):
        self.topic_category_list = { }
        self.source_category_list = { }
        self.status_category_list = { }
        self.style = None
    def with_style(self, style):
        self.style = style
        return self
    def register_page(self, page, categories):
        topic = categories.topic_category
        source = categories.source_category
        status = categories.status_category
        name = os.path.basename(page.name)
        topic_name = name+" ("+source+")"
        source_name = name+" ("+topic+")"
        status_name = name+" ("+source+" "+topic+")"
        if topic not in self.topic_category_list:
            self.topic_category_list[topic] = { }
        if name not in self.topic_category_list[topic]:
            self.topic_category_list[topic][topic_name] = page
        if source not in self.source_category_list:
            self.source_category_list[source] = { }
        if name not in self.source_category_list[source]:
            self.source_category_list[source][source_name] = page
        if status not in self.status_category_list:
            self.status_category_list[status] = { }
        if name not in self.status_category_list[status]:
            self.status_category_list[status][status_name] = page
    def html_info(self, text, topic):
        if self.style:
            info = self.style.html_info(topic)
            if not info: return text
            return (
                '<table cellpadding="0" width="100%"><tr valign="top">'+
                '<td>'+n+text+'</td>'+
                '<td valign="right">'+n+info+'</td>'+n+'</tr></table>')
    def generate_htmlpages(self): # yield list
        for topic in self.topic_category_list:
            subtitle = ""
            if self.style: subtitle = (
                '<blockquote><em>'+self.style.subtitle(topic)+
                "</em></blockquote>"+n)
            htmlpage = "<ul>"+n
            for name in sorted(self.topic_category_list[topic].keys()):
                page = self.topic_category_list[topic][name]
                htmlpage += '<li><a href="../'+page.name+htm+'">'
                htmlpage += name+'</a></li>'+n
            htmlpage += "</ul>"+n
            index = HtmlIndexPage(topic.replace(" ","_")+"/index")
            index.title = topic
            index.htmlpage = "<h2>"+index.title+"</h2>"+n+subtitle+htmlpage+n
            index.subtype = "topic"
            index.htmlpage = self.html_info(index.htmlpage, topic)
            yield index
        for source in self.source_category_list:
            subtitle = ""
            if self.style: subtitle = (
                '<blockquote><em>'+self.style.subtitle(source)+
                "</em></blockquote>"+n)
            htmlpage = "<ul>"+n
            for name in sorted(self.source_category_list[source].keys()):
                page = self.source_category_list[source][name]
                htmlpage += '<li><a href="../'+page.name+htm+'">'
                htmlpage += name+'</a></li>'+n
            htmlpage += "</ul>"+n    
            index = IndexPage(source.replace(" ","_")+"/index")
            index.title = source+"'s macros"
            index.htmlpage = "<h2>"+index.title+"</h2>"+n+subtitle+htmlpage+n
            index.subtype = "source"
            index.htmlpage = self.html_info(index.htmlpage, source)
            yield index
        for status in self.status_category_list:
            subtitle = ""
            if self.style: subtitle = (
                '<blockquote><em>'+self.style.subtitle(status)+
                "</em></blockquote>"+n)
            htmlpage = "<ul>"+n
            for name in sorted(self.status_category_list[status].keys()):
                page = self.status_category_list[status][name]
                htmlpage += '<li><a href="../'+page.name+htm+'">'
                htmlpage += name+'</a></li>'+n
            htmlpage += "</ul>"+n    
            index = IndexPage(status.replace(" ","_")+"/index")
            index.title = status+" macros"
            index.htmlpage = "<h2>"+index.title+"</h2>"+n+subtitle+htmlpage+n
            index.subtype = "status"
            index.htmlpage = self.html_info(index.htmlpage, status)
            yield index
    def overview_htmlpage(self):
        today = date.today()
        macros_tgz = "ac-archive-macros-"+today.strftime("%Y.%m%d")+".tar.bz2"
        index = HtmlIndexPage("macros")
        html = "<H2> AC Autoconf Macro Archive Index </H2>"+n
        html += "<dl>"+n
        for topic in sorted(self.topic_category_list.keys()):
            html += '<DT>'+topic+'</DT><dd><small>'+n
            comma = ""
            for name in self.topic_category_list[topic]:
                html += comma+" "+name
                comma = ","
            html += '</small></dd>'+n
        html += "</dl>"+n
        html += '<p align="center">All these macros are in '
        html += '<a href="'+macros_tgz+'">'+macros_tgz+'</a>'
        html += ' macros only tarball.</p>'+n
        index.htmlpage = html
        index.title = "Index"
        return index
