#! /usr/bin/env python
# -*- coding: utf-8 -*- 100 -*-
# @creator (C) 2003 Guido U. Draheim
# @license http://creativecommons.org/licenses/by-nc-sa/2.0/de/

import string
import os.path
from match import Match
import htm2dbk

n = "\n"

def escape_html(text): # text
    return text.replace("&","&amp;").replace("<","&lt;").replace(">","&gt;")
def escape_blanks(text):
    return text.replace(" ","&nbsp;")
def latin1_utf8(text):
    """ fixup for some macros with non-utf8 codes """
    return (text
            & Match("(\\w)\xE5(\\w)") >> "\\1å\\2" 
            & Match("(\\w)\xE8(\\w)") >> "\\1è\\2" 
            & Match("(\\w)\xFC(\\w)") >> "\\1ü\\2")

class MacroFile:
    def __init__(self, filename = None, into = None):
        self.html = None
        self.macro = ""
        self.author = ""
        self.synopsis = ""
        self.summary = ""
        self.license = ""
        self.copyright = ""
        self.version = ""
        self.category = ""
        self.obsoleted = ""
        self.documentation = ""
        if filename: self.scan(filename, into)
    def pagetype(self):
        return "macro"
    def html_page(self):
        return self.html
    def scan(self, filename, into = None):
        self.orig = filename
        self.text = latin1_utf8(file(filename).read())
        name = filename[:-3] # s/.m4//
        self.name = name
        self.into = into
        self.category = into
    def scan_text(self, text, into = ""):
        self.orig = into
        self.text = latin1_utf8(text)
        self.name = into
        self.into = into
        self.category = into
        return self
    def html_info(self):
        return "Macro - "+self.page.name
    def html_text(self):
        """ default html page """
        return ("<H1>"+self.name+"</H1>"+"\n"+"<pre>"+self.macro+"</pre>\n"+
                "<br><small>create_html</html>")
    def parse(self):
        "parse_macro()"
        x = Match()
        last = ""
        ends = ""
        self.category = "" # ???
        for line in self.text.split("\n"):
            ## sys.stderr.write("!")
            if last == "macro":
                self.macro += escape_html(line)+"\n"
            elif line & x(r"dnl\s+\@(?:synopsis|[*])\s+(.*)\s*$","i"):
                if last != "synopsis": self.documentation += ends ; ends = ""
                last = "synopsis"
                if len(self.synopsis): self.synopsis += "<br />\n"
                self.synopsis += escape_html(x[1])+"\n"
            elif line & x(r"dnl\s+\@(?:summary|[=])\s+(.*)\s*$","i"):
                if last != "summary": self.documentation += ends ; ends = ""
                last = "summary"
                if len(self.summary): self.summary += "<br />\n"
                self.summary += escape_html(x[1])+"\n"
            elif line & x(r"^dnl\s+\@(?:author|[;])\s+(.*)\s*$","i"):
                if last != "author": self.documentation += ends ; ends = ""
                last = "author"
                if len(self.synopsis): self.synopsis += ", \n"
                self.author += escape_html(x[1])+"\n"
            elif line & x(r"^dnl\s+\@(?:license|[!])\s+(.*)\s*$","i"):
                if last != "license": self.documentation += ends ; ends = ""
                last = "license"
                self.license += escape_html(x[1])+"\n"
            elif line & x(r"^dnl\s+\@(?:copyright|[(][Cc][)])\s+(.*)\s*$","i"):
                if last != "copyright": self.documentation += ends ; ends = ""
                last = "copyright"
                self.copyright += escape_html(x[1])+"\n"
            elif line & x(r"^dnl\s+\@version\s+(.*)\s*$","i"):
                if last != "version": self.documentation += ends ; ends = ""
                last = "version"
                part = x[1]
                if part & x(r"^\$[I][d]: [^ ]+ ([^ ]+) ([^ ]+)","i"):
                    self.version = escape_html(x[1])+" ("+escape_html(x[2])+")"
                elif part & x(r"^\$[I][d]:(.*)\$(.*)","i"):
                    self.version = escape_html(x[1]+" "+x[2])
                else:
                    self.version = escape_html(part)
            elif line & x(r"^dnl\s+\@(?:category|[\,])\s+(.*)\s*$","i"):
                self.category = x[1]
            elif line & x(r"^dnl\s+\@?(obsoleted[:\s].*)\s*$","i"): #obsoleted
                self.obsoleted = x[1]
                if last != "enum":
                    self.documentation += ends+"<ul>\n"; ends = "</ul>\n"
                    last = "enum"
                self.documentation += "<li><p><b>"+escape_html(x[1])+"</b></p></li>"
            elif line & x(r"^dnl\s+([*-]\s.*)\s*$","i"):            #enumeration
                if last != "enum":
                    self.documentation += ends+"<ul>\n"; ends = "</ul>\n"
                    last = "enum"
                self.documentation += "<li><p>"+escape_html(x[1])+"</p></li>\n"
            elif line & x(r"^dnl\s\s(.*)\s*$","i"):                 #verbatim
                if last != "screen":
                    self.documentation += ends+"<pre>"; ends = "</pre>"
                    last = "screen"
                self.documentation += escape_html(x[1])+"\n"
            elif line & x(r"^dnl\s*(.*)\s*$","i"):
                if not len(x[1]):
                    if last != "para":
                        self.documentation += ends+"<p>" ; ends = "</p>"
                        last = "para"
                    else:
                        self.documentation += "</p><p>"
                else:
                    if last != "para":
                        self.documentation += ends+"<p>" ; ends = "</p>"
                        last = "para"
                    self.documentation += escape_html(x[1])+"\n"
            else:
                if last != "macro": self.documentation += ends ; ends = ""
                last = "macro"
                self.macro = escape_html(line)+"\n"
        if ends: self.documentation += ends ; ends = ""

class MacroLicensing:
    def std_license(self, text):
        T = text.strip().lower()
        if T == "allpermissive":
            return self.AllPermissive()
        elif T == "gplwithacexception":
            return self.GPLWithACException()
        else:
            return T
    def AllPermissive(self):
        return """AllPermissive <br /><small><em>Copying and distribution of this file, with or
        without modification, are permitted in any medium without royalty provided the copyright
        notice and this notice are preserved. Users of this software should generally follow the
        principles of the <a href="http://www.opensource.org/licenses/mit-license.php">
        MIT License</a> includings its disclaimer. </em></small>"""
    def GPLWithACException(self):
        return """GPLWithACException <br /><small><em>This program is free software; you can
        redistribute it and/or modify it under the terms of the GNU General Public License
        as published by the Free Software Foundation. As a special exception, the respective
        Autoconf Macro's copyright owner gives unlimited permission to copy, distribute and modify
        the configure scripts that are the output of Autoconf when processing the Macro. You
        need not follow the terms of the
        <a href="http://www.opensource.org/licenses/gpl-license.php">
        GNU General Public License</a> when using or distributing such scripts</em></small>"""
class MacroSource:
    def std_source(self, text):
        T = text.strip().lower()
        if T == "ac-archive":
            return """<a href="http://autoconf-archive.cryp.to">cryp.to ac-archive</a>"""
        else:
            return T
        
class HtmlMacroPage:
    def __init__(self, page, categories):
        self.page = page
        self.categories = categories
        self.title = None
        self.licensing = MacroLicensing()
        self.macrosource = MacroSource()
    def std_license(self, text):
        return self.licensing.std_license(text)
    def std_source(self, text):
        return self.macrosource.std_source(text)
    def pagetype(self):
        return self.page.pagetype()
    def get_name(self):
        return self.page.name
    def get_title(self):
        if not self.title:
            if not self._gen_html(): return None
        return self.title
    def html_info(self):
        if not self.title:
            if not self._gen_html(): return None
        return self.htmlpageinfo
    def html_text(self):
        if not self.title:
            if not self._gen_html(): return None
        return self.htmlpage
    def _gen_html(self):
        if not self.title:
            self.gen_html()
        return True
    def gen_html(self):
        page = self.page
        self.title = page.name.replace("/"," - ")
        self.htmlpageinfo = "#"
        self.htmlpage = self.htmlpage_header(page)
        self.htmlpage += """<dl>"""+n
        if page.synopsis and page.summary:
            self.htmlpage += """<DT>Synopsis</DT>"""+n
            self.htmlpage += ("<DD><code>"+page.synopsis+"</code>"+n
                              +"<p>"+page.summary+"</p></DD>"+n)
        elif page.synopsis:
            self.htmlpage += """<DT>Synopsis</DT>"""+n
            self.htmlpage += "<DD><pre>"+page.synopsis+"</pre></DD>"+n
        if page.version:
            self.htmlpage += """<DT>Version</DT>"""+n
            self.htmlpage += "<DD><p>"+page.version+"</p></DD>"+n
        if page.author:
            self.htmlpage += """<DT>Author</DT>"""+n
            self.htmlpage += "<DD><p>"+page.author+"</p></DD>"+n
        if page.copyright:
            self.htmlpage += """<DT>Copyright</DT>"""+n
            self.htmlpage += "<DD><p>"+page.copyright+"</p></DD>"+n
        if page.license:
            self.htmlpage += """<DT>License</DT>"""+n
            self.htmlpage += "<DD><p>"+self.std_license(page.license)+"</p></DD>"+n
        if self.categories.topic_category:
            self.htmlpage += """<DT>Category</DT>"""+n
            self.htmlpage += "<DD><p>"+self.std_source(self.categories.source_category)
            self.htmlpage += "'s "+self.categories.topic_category
            self.htmlpage += " ("+self.categories.status_category+")</p></DD>"+n
        if page.documentation:
            self.htmlpage += """<DT>Documentation</DT>"""+n
            self.htmlpage += "<DD>"+page.documentation+"</DD>"+n
        self.htmlpage += """<DT>M4 Source Code</DT>"""+n
        self.htmlpage += "<DD><pre>"+escape_html(page.macro)+"</pre></DD>"+n
        # self.htmlpage += """<DT>Original Text</DT>"""+n
        # self.htmlpage += "<DD><pre>"+escape_html(page.text)+"</pre></DD>"+n
        self.htmlpage += "</dl>"+n
    back_to_mainpage="Back&nbsp;to&nbsp;the Main&nbsp;Page."
    download_m4src="Download the&nbsp;M4&nbsp;Source."
    def htmlpage_header(self, page):
        return '''
        <table width="100%"><tr>
        <td align="center">
        <a href="'''+os.path.basename(page.orig)+'''">'''+self.download_m4src+'''</a>
        </td><td align="center">
        <h2>'''+os.path.basename(page.name)+'''</h2>
        </td><td align="center">
        <a href="../">'''+self.back_to_mainpage+'''</a>
        </td></tr></table><hr width="80%" align="center" />
        '''    

class XmlMacroPage:
    def __init__(self, page, o):
        self.page = page
        self.o = o
        self.title = None
        self.manvolnum = "7"
        self.productname = ""
        if o.package: self.productname = o.package
        self.productversion = ""
        if o.version: self.productversion = o.version
    def pagetype(self):
        return self.page.pagetype()
    def get_name(self):
        return self.page.name
    def get_title(self):
        if not self.title:
            if not self._gen_xml(): return None
        return self.title
    def xml_info(self):
        if not self.title:
            if not self._gen_xml(): return None
        return self.xmlpageinfo
    def xml_text(self):
        if not self.title:
            if not self._gen_xml(): return None
        return self.xmlpage
    def _gen_xml(self):
        if not self.title:
            self.gen_xml()
        return True
    def as_purpose(self, text):
        line = string.lower(text.replace("_"," "))
        return ((" "+line+" ")
                .replace(" h ",".h ")
                .replace(" ac "," ")
                .replace(" ax "," ")
                .replace(" creat "," creating ")
                .replace(" check "," checking ")
                .replace(" prog "," check for program "))
    def get_purpose(self, text):
        if self.page.summary:
            return self.page.summary.strip()
        else:
            return self.as_purpose(text)
    def set_title(self, name):
        base = os.path.basename(name)
        if base == name:
            self.title = name
        else:
            domain = os.path.dirname(name).replace("/","-")
            if domain == "ac-archive": domain = "m4"
            self.title = base+"."+domain
    def list_func_params(self, text):
        m = Match(r"\s*(\w+)(\s.*|\[.*|\(.*)")
        for item in text.split("<br />"):
            callname = self.page.synopsis ; callparams = ""
            if item & m:
                callname = m[1].strip() ; callparams = m[2].strip()
            else:
                print "# could not parse synopsis: '"+text+"' in "+self.page.name
            yield callname, callparams.strip()
    def para(self, text):
        if not text.startswith("<para>"):
            return "<para>"+text+"</para>"
        return text
    def screen(self, text):
        if not text.startswith("<screen>"):
            return "<screen>"+text+"</screen>"
        return text
    def gen_xml(self):
        """ creates a single refentry """
        page = self.page
        self.set_title(page.name)
        self.xmlpageinfo = "#"
        default_purpose = os.path.basename(page.name)
        text = "<refentry>\n"
        text += "<refentryinfo>\n"
        text += " <date>"+self.productversion+"</date>\n"
        text += " <productname>"+self.productname+"</productname>\n"
        text += "</refentryinfo>\n"
        text += "<refmeta>\n"
        text += " <refentrytitle>"+self.title+"</refentrytitle>\n"
        text += " <manvolnum>"+self.manvolnum+"</manvolnum>\n"
        text += "</refmeta>\n"
        text += "<refnamediv>\n"
        text += " <refname>"+self.title+"</refname>\n"
        text += " <refpurpose>"+self.get_purpose(default_purpose)+"</refpurpose>\n"
        text += "</refnamediv>\n"
        text += "<refsynopsisdiv>\n"
        text += " <funcsynopsis>\n"
        if page.version:
            text += "  <funcsynopsisinfo>"+page.version+"</funcsynopsisinfo>\n"
        for callname, callparams in self.list_func_params(page.synopsis):
            text += "  <funcprototype>\n"
            text += "   <funcdef>"+callname+"</funcdef>\n"
            if callparams:
                text += "   <paramdef>"+callparams+"</paramdef>\n"
            else:
                text += "   <void/>\n"
            text += "  </funcprototype>\n"
        text += " </funcsynopsis>\n"
        text += "</refsynopsisdiv>\n"
        if page.documentation:
            text += "<refsect1><title>Description</title>\n"
            text += htm2dbk.html2docbook(page.documentation)
            text += "</refsect1>\n"
        if page.author:
            text += "<refsect1><title>Author</title>\n"
            text += self.para(page.author)
            text += "</refsect1>\n"
        if page.copyright:
            text += "<refsect1><title>Copyright</title>\n"
            text += self.para(page.copyright)
            text += "</refsect1>\n"
        if page.license:
            text += "<refsect1><title>License</title>\n"
            text += self.para(page.license)
            text += "</refsect1>\n"
        if page.macro:
            text += "<refsect1><title>M4 Source code</title>\n"
            text += self.screen(page.macro) # escape_html(page.macro)
            text += "</refsect1>\n"
        text += "</refentry>\n"
        self.xmlpage = text
        
