#! /usr/bin/env python
# -*- coding: utf-8 -*-
# @creator (C) 2003 Guido U. Draheim
# @license http://creativecommons.org/licenses/by-nc-sa/2.0/de/

import re

# ---------------------------------------------------------- Regex Match()
# beware, stupid python interprets backslashes in replace-parts only partially!
class MatchReplace:
    """ A MatchReplace is a mix of a Python Pattern and a Replace-Template """
    def __init__(self, matching, template):
        self.matching = matching
        self.template = template
        self.count = 0
    def __and__(self, string):
        text, self.matching.replaced = \
              self.matching.regex.subn(self.template, string, self.count)
        return text
    def __rand__(self, string):
        text, self.matching.replaced = \
              self.matching.regex.subn(self.template, string, self.count)
        return text
    def __iand__(self, string):
        string, self.matching.replaced = \
                self.matching.regex.subn(self.template, string, self.count)
        return string
    def __rshift__(self, count):
        self.count = count ; return self
    def __rlshift__(self, count):
        self.count = count ; return self

class Match(str):
    """ A Match is actually a mix of a Python Pattern and MatchObject """
    def __init__(self, pattern = None, flags = None):
        Match.__call__(self, pattern, flags)
    def __call__(self, pattern, flags = None):
        assert isinstance(pattern, str) or pattern is None
        assert isinstance(flags, str) or flags is None
        str.__init__(self, pattern)
        self.replaced = 0 # set by subn() inside MatchReplace
        self.found = None # set by search() to a MatchObject
        self.pattern = pattern
        if pattern is not None:
            if flags:
                self.regex = re.compile("(?"+flags+")"+self.pattern)
            else:
                self.regex = re.compile(self.pattern)
        return self
    def __truth__(self):
        return self.found is not None
    def __and__(self, string):
        self.found = self.regex.search(string)
        return self.__truth__()
    def __rand__(self, string):
        self.found = self.regex.search(string)
        return self.__truth__()
    def __div__(self, string):
        return self.regex.finditer(string)
    def __rdiv__(self, string):
        return self.regex.finditer(string)
    def __floordiv__(self, string):
        return self.regex.findall(string)
    def __rfloordiv__(self, string):
        return self.regex.findall(string)
    def __rshift__(self, template):
        return MatchReplace(self, template)
    def __rlshift__(self, template):
        return MatchReplace(self, template)
    def __getitem__(self, index):
        assert self.found is not None
        return self.found.group(index)
