#! /usr/bin/env python
# -*- coding: utf-8 -*-
# @creator (C) 2003 Guido U. Draheim
# @license http://creativecommons.org/licenses/by-nc-sa/2.0/de/

from datetime import date

htm = ".html"
n = "\n"
br = "<br />"

class HtmlNavBar:
    captions = {
        "macros" : "Macro Index",
        "AM_Support/index" : "AM Support",
        "C++_Support/index" : "C++ Support",
        "C_Support/index" : "C Support",
        "Fortran_Support/index" : "Fortran Support",
        "Java_Support/index" : "Java Support",
        "Cross_Compilation/index" : "Cross Compilation",
        "Installed_Packages/index" : "Installed Packages",
        "Miscellaneous/index" : "Miscellaneous",
        "System_Headers/index" : "System Headers",
        "System_Utilities/index" : "System Utilities",
        "search" : "search index" }
    topics0 = [ "macros" ]
    topics1 = [ "AM_Support/index",
                "C++_Support/index",
                "C_Support/index",
                "Fortran_Support/index",
                "Java_Support/index"]
    topics2 = [ "Cross_Compilation/index",
                "Installed_Packages/index",
                "Miscellaneous/index",
                "System_Headers/index",
                "System_Utilities/index"]
    status1 = [ ]
    status2 = [ "search" ]
    def is_obsoleted(self, page):
        return page.name.find("bsolete") > 0
    def __init__(self):
        self.pages = { }
    def register_page(self, page):
        self.pages[page.name] = page
    def html_logo(self, root):
        return '''
        <!--logo:-->
        <center>
        <big><big><a href="http://ac-archive.sf.net">
        <font color="#4000C0"><b>AC</b><small>-</small><i>Archive</i></font>
        </a></big></big>
        <br /><small>Autoconf Macro Archive</small></center>
        <!--logo/-->
        '''
    def html_project(self, root):
        return '''
        <!--site:-->
        <A href="http://ac-archive.sf.net">ac-archive.sf.net:</A>
        - <A href="http://sf.net/projects/ac-archive/">Project</A>
        <A href="http://sf.net/cvs/?group_id=32081">CVS</A>
        - <A href="http://sf.net/project/showfiles.php?group_id=32081">
        Download</A><!--site/-->
        '''
    def _format(self, page, caption, root):
        caption = caption.replace("ac-archive's", "archive")
        return '<a href="'+root+page+htm+'">'+caption+'</a>'+br
    def html_dirs(self, root):
        done = []
        txt = "<!--dirs:-->"
        for name in self.topics0:
            if name in self.pages:
                page = self.pages[name]
                txt += n + self._format(name, self.captions[name], root)
                done += [ page ]
            else:
                txt += n + self._format(name, self.captions[name], root)
        for name in self.topics1:
            if name in self.pages:
                page = self.pages[name]
                txt += n + "- " + self._format(name, self.captions[name], root)
                done += [ page ]
        for name in self.topics2:
            if name in self.pages:
                page = self.pages[name]
                txt += n + "- " + self._format(name, self.captions[name], root)
                done += [ page ]
        for name in sorted(self.pages.keys()):
            page = self.pages[name]
            if page.subtype != "topic": continue
            if self.is_obsoleted(page): continue
            if page not in done:
                txt += n + "- " + self._format(page.name, page.title, root)
                done += [ page ]
        for name in sorted(self.pages.keys()):
            page = self.pages[name]
            if page.subtype != "source": continue
            if page not in done:
                txt += n + "- " + self._format(page.name, page.title, root)
                done += [ page ]
        for name in self.status1:
            if name in self.pages:
                page = self.pages[name]
                txt += n + "- " + self._format(name, self.captions[name], root)
                done += [ page ]
            else:
                txt += n + "- " + self._format(name, self.captions[name], root)
        for name in sorted(self.pages.keys()):
            page = self.pages[name]
            if page.subtype != "status": continue
            if page not in done:
                txt += n + "- " + self._format(page.name, page.title, root)
                done += [ page ]
        for name in self.status2:
            if name in self.pages:
                page = self.pages[name]
                txt += n + "- " + self._format(name, self.captions[name], root)
                done += [ page ]
            else:
                txt += n + "- " + self._format(name, self.captions[name], root)
        txt += n + "<!--dirs/-->"
        return txt
    def html_docs(self, root):
        return '''
        <!--docs:-->
        <a href="'''+root+'''doc/introduction.html">Documentation</a><br />
        - <a href="'''+root+'''doc/contribute.html">Contribute!</a><br />
        - <a href="'''+root+'''doc/history.html">History</a><br />
        - <a href="'''+root+'''doc/acincludedir.html">acincludedir&nbsp;m4</a><br />
        - <a href="'''+root+'''doc/acinclude.html">acinclude (tool)</a><br />
        - <a href="'''+root+'''doc/howto.html">macro howto</a><br />
        - <a href="'''+root+'''doc/tricks.html">ax tricks</a><br />
        - <a href="'''+root+'''doc/maintainers.html">maintainers</a><br />
        - <a href="'''+root+'''doc/copyright.html">License</a><br />
        - <a href="'''+root+'''doc/else.html">Topics</a><br />
        <!--docs/-->
        '''
    def html_copyright(self, root):
        today = date.today()
        return '''
        <small><i>generated...</i><br />
        <center>'''+str(today)+''' </center></small><br />
        <b><a href="'''+root+'''copyright.html">(C) ''' \
    +str(today.year)+'''</a>  <a href="mailto:guidod[@]gmx.de">guidod</a></b>
        '''
    def html(self, root):
        return (self.html_logo(root)+"\n<hr />\n"+
                self.html_project(root)+"\n<hr />\n"+
                self.html_dirs(root)+"\n<hr />\n"+
                self.html_docs(root)+"\n<hr />\n"+
                self.html_copyright(root))
                

if __name__ == "__main__":
    x = HtmlNavBar()
    print x.html()
    
