#! /usr/bin/env python
# -*- coding: utf-8 -*- 100 -*- python -*-
# @creator (C) 2006 Guido U. Draheim
# @license http://creativecommons.org/licenses/by-nc-sa/2.0/de/
"""
Scan all subdirectories and create the apropriate index list.
Scan the arguments and build a proper website from the input.
"""

import re
import sys
import os
import os.path
import glob
# sys.path += [ "@site_packages@" ]
from ac_archive.os_ import *
from ac_archive.warn_ import *
from ac_archive.sorted_ import *
from ac_archive.options import *
from ac_archive.match import *
from ac_archive.subdirs import *
from ac_archive.navbar import *
from ac_archive.macrofile import *
from ac_archive.docusfile import *
from ac_archive.extrafile import *
from ac_archive.spellcheck import *
from ac_archive.indexgen import *
from ac_archive.htmlgen import *
from ac_archive.docbookdocument import *
from ac_archive.extras import *

# -------------------------------------------------------------------------------- options / style
o = Options()

o.top_builddir = "."
o.topx = "macros"
n  = "\n"
br = "<br />\n"

style = AskStyleExtras()

# ------------------------------------------------------------------------------------ input scan

class Page:
    def __init__(self, src):
        self.src = src
        self.htmlpage = None
        self.xmlpage = None
        self.categories = None
    def pagetype(self):
        if self.src:
            return self.src.pagetype()
        elif self.htmlpage:
            return self.htmlpage.pagetype()
        elif self.xmlpage:
            return self.htmlpage.pagetype()
        else:
            return None
    def xml_text(self):
        if not self.xmlpage: return None
        return self.xmlpage.xml_text()

class OutputFiles:
    def __init__(self):
        self.files = {}
    def get_src(self, name):
        return self.files[name].src
    def add_src(self, name, page):
        self.files[name] = Page(page)
    def add_htmlpage(self, name, page):
        item = Page(None)
        item.htmlpage = page
        self.files[name] = item
    def pages(self, pagetype):
        for page in self.files.values():
            if page.pagetype() == pagetype:
                yield page
    def src_pages(self, pagetype):
        for page in self.files.values():
            if page.pagetype() == pagetype:
                yield page.src
    def sorted_pages(self, pagetype):
        for name in sorted(self.files.keys()):
            page = self.files[name]
            if page.pagetype() == pagetype:
                yield page
    def sorted_src_pages(self, pagetype):
        for name in sorted(self.files.keys()):
            page = self.files[name]
            if page.pagetype() == pagetype:
                yield page.src

out = OutputFiles()

class AdminFile:
    def __init__(self, filename, into = None):
        self.orig = filename
        self.into = into
        self.admintext = file(filename).read()
    def pagetype(self):
        return "admin"
class SrcFiles:
    def __init__(self, outputfiles):
        self.outputfiles = outputfiles
        self.directories = []
    def add_directory(self, dirname):
        self.directories += [ dirname ]
    def scan_htm(self, filename, into = None):
        target = filename[:-len(".htm")]
        self.outputfiles.add_src(target, DocusFile(filename))
    def scan_m4(self, filename, into = None):
        target = filename[:-len(".m4")]
        self.outputfiles.add_src(target, MacroFile(filename, into))
    def scan_txt(self, filename, into = None):
        target = filename[:-len(".txt")]
        self.outputfiles.add_src(target, ExtraFile(filename, into))
    def scan_log(self, filename, into = None):
        print filename,"(skip)", ; return # SKIP
        target = filename[:-len(".log")]
        self.outputfiles.add_src(target, AdminFile(filename, into))
       
def scan_subdirs(dirlist):
    "scan_subdirs()"
    global out
    if not dirlist: return None
    page = IndexPage(o.topx)
    page.title = "The Autoconf Macro Archive!"
    out.add_src(o.topx, page)
    print "register_default_subdirs()"
    subdirfiles = ScanSubdirs(SrcFiles(out), o)
    for topdir in dirlist:
        subdirfiles.scan(topdir)
    return subdirfiles

def scan_args(argv):
    files = SrcFiles(out)
    for arg in argv:
        try:
            if o.scan(arg): continue
            if arg.endswith(".htm"): files.scan_htm(arg)
            if arg.endswith(".m4"):  files.scan_m4(arg)
            if arg.endswith(".txt"): files.scan_txt(arg)
            if arg.endswith(".log"): files.scan_log(arg)
            if os.path.isdir(arg): files.add_directory(arg)
        except IOError, e:
            print arg,":", e
    return files.directories
# ------------------------------------------------------------------------------------ parse / gen

class ParseAndGen:
    def __init__(self, o):
        self.o = o
        self.index = IndexGenerator()
        self.html_navbar = HtmlNavBar()
        self.aspell = SpellChecker()
    #
    def with_style(self, style):
        if self.index:
            self.index.with_style(style)
        return self
    def parse_docus(self):
        "parse_docus()"
        global out
        if not o.quiet: sys.stderr.write("docus:")
        for page in out.pages("docus"):
            if not o.quiet: sys.stderr.write(".")
            page.src.parse()
        if not o.quiet: sys.stderr.write(n)
    def parse_macros(self):
        "parse_macros()"
        global out
        if not o.quiet: sys.stderr.write("macros:")
        for page in out.pages("macro"):
            if not o.quiet: sys.stderr.write(".")
            page.src.parse()
        if not o.quiet: sys.stderr.write(n)
    def parse_spellcheck(self):
        "parse_spellcheck()"
        global out
        if not o.quiet: sys.stderr.write("spells:")
        for page in out.pages("macro"):
            if not o.quiet: sys.stderr.write(".")
            self.aspell.spellcheck(page.src)
        if not o.quiet: sys.stderr.write(n)
    def macro_categories(self):
        print "macro_categories()"
        for page in out.sorted_pages("macro"):
            page.categories = IndexCategory(page.src)
            self.index.register_page(page.src, page.categories)
    def gen_index_pages(self):
        print "gen_index_pages()"
        for page in self.index.generate_htmlpages():
            out.add_htmlpage(page.name, page)
            self.html_navbar.register_page(page)
    def parse_extra_defs(self):
        print "parse_extra_defs()"
        pass
    def fixup_extra_defs(self):
        print "fixup_extra_defs()"
        pass
    def search_index_words(self):
        print "search_index_words()"
        pass
    def index_categories(self):
        print "index_categories()"
        pass
    def mark_navi_pages(self):
        print "mark_navi_pages()"
        pass
    def gen_all_index_html(self):
        print "gen_all_index_html()"
        pass
    def gen_index_frontpage(self):
        page = self.index.overview_htmlpage()
        out.add_htmlpage(page.name, page)
    def gen_search_index_page(self, filename):
        page = HtmlIndexPage(filename)
        page.htmlpage = self.aspell.html_spelled()
        out.add_htmlpage(page.name, page)
    def gen_all_docus_html(self):
        for page in out.sorted_pages("docus"):
            page.htmlpage = HtmlDocusPage(page.src, self.o, style)
    def gen_all_macro_html(self):
        for page in out.sorted_pages("macro"):
            page.htmlpage = HtmlMacroPage(page.src, page.categories)
    def gen_all_macro_xml(self):
        for page in out.sorted_pages("macro"):
            page.xmlpage = XmlMacroPage(page.src, self.o)
    def parse_and_gen(self):
        self.parse_docus()
        self.parse_macros()
        self.parse_spellcheck()
        self.parse_extra_defs()
        self.fixup_extra_defs()
        self.search_index_words()
        self.gen_search_index_page("search")
        self.macro_categories()
        self.index_categories()
        self.mark_navi_pages()
        self.gen_index_pages()
        self.gen_index_frontpage()
        self.gen_all_index_html()
        self.gen_all_macro_html()
        self.gen_all_docus_html()
        self.gen_all_macro_xml()
    
# ------------------------------------------------------------------------------------ main steps
def help():
    print "ac_archive_gendocs [options]... [directory]... [docfiles]..."
    print " scan the directories for m4 files to be documented"
    print " and append doc files in the html report series"
    print "options:"
    print "--package=name set package to be reported"
    print "--version=name set version to be reported"
    print "--release=name set release to be reported"
    print "--output=directory for the html report files"
    print "--topx=name main topic for the macro summary"
    print "--quiet do not show progess messages"
    print "--test try to build as a single file"
    print "--docbook=ext filename extension for docbook files"
    print "--html=ext filename extension for html files"
    print "note:"
    print "  creates also <package>-<topx>.<docbook>"
    print "  i.e. default ac_archive-macros.docbook"
    print "                               (C) Guido U. Draheim"

def main(argv):
    o.version = "0.5.66"
    o.package = "ac-archive"
    o.topic = "macros"
    # print style.expand("<DT>foo</DT>")
    dirlist = scan_args(argv)
    if not dirlist: help() ; exit(0)
    scan_subdirs(dirlist)
    comp = ParseAndGen(o).with_style(style)
    comp.parse_and_gen()
    HtmlGenerator(o).with_navbar(comp.html_navbar).with_style(style).generate_output(out)
    doc = DocbookDocument(o)
    for page in out.pages("macro"):
        doc.add(page)
    doc.save(o.package+"-"+o.topic)
    
if __name__ == "__main__":
    main(sys.argv[1:])
    
