#! /usr/bin/env python
# -*- coding: utf-8 -*- python -*-
# @creator (C) 2007 Guido U. Draheim
# @license http://creativecommons.org/licenses/by-nc-sa/2.0/de/
"""
Scan the arguments and build a proper docbook xml reference page from it.
"""

import re
import sys
import os
import os.path
import glob
# sys.path += [ "@site_packages@" ]
from ac_archive.os_ import *
from ac_archive.warn_ import *
from ac_archive.sorted_ import *
from ac_archive.options import *
from ac_archive.match import *
from ac_archive.subdirs import *
from ac_archive.navbar import *
from ac_archive.macrofile import *
from ac_archive.docusfile import *
from ac_archive.extrafile import *
from ac_archive.spellcheck import *
from ac_archive.indexgen import *
from ac_archive.htmlgen import *
from ac_archive.docbookdocument import *
from ac_archive.extras import *

cgipath = "http://guidod.homelinux.org/ac-archive/"

# ---------------------------------------------------------- options / style
o = Options()

o.top_builddir = "."
o.topx = "macros"
n  = "\n"
br = "<br />\n"

style = AskStyleExtras()

class Page:
    def __init__(self, src):
        self.src = src
        self.htmlpage = None
        self.xmlpage = None
        self.categories = None
    def pagetype(self):
        if self.src:
            return self.src.pagetype()
        elif self.htmlpage:
            return self.htmlpage.pagetype()
        elif self.xmlpage:
            return self.htmlpage.pagetype()
        else:
            return None
    def xml_text(self):
        if not self.xmlpage: return None
        return self.xmlpage.xml_text()

class OutputFiles:
    def __init__(self):
        self.files = {}
    def add_src(self, name, page):
        self.files[name] = Page(page)
    def pages(self, pagetype):
        for page in self.files.values():
            if page.pagetype() == pagetype:
                yield page
    def sorted_pages(self, pagetype):
        for name in sorted(self.files.keys()):
            page = self.files[name]
            if page.pagetype() == pagetype:
                yield page

out = OutputFiles()

class SrcFiles:
    def __init__(self, outputfiles):
        self.outputfiles = outputfiles
        self.directories = []
    def add_directory(self, dirname):
        self.directories += [ dirname ]
    def scan_m4(self, filename, into = None):
        target = filename[:-len(".m4")]
        self.outputfiles.add_src(target, MacroFile(filename, into))
    def scan_m4_text(self, text, into):
        self.outputfiles.add_src(into, MacroFile().scan_text(text, into))

def scan_args(argv):
    files = SrcFiles(out)
    for arg in argv:
        try:
            if o.scan(arg): continue
            if arg.endswith(".m4"):  files.scan_m4(arg)
            if os.path.isdir(arg): files.add_directory(arg)
        except IOError, e:
            print arg,":", e
    return files.directories
def scan_cgi(argv):
    import cgi
    form = cgi.FieldStorage()
    if form.has_key("text"):  o.text = form['text'].value
    if form.has_key("format"):  o.format = form['format'].value
    if form.has_key("title"):  o.title = form['title'].value
    if not o.text: o.format = ""; return
    if not o.title: o.title = "macro"
    files = SrcFiles(out)
    files.scan_m4_text(o.text, o.title)

class ParseAndGen:
    def __init__(self, o):
        self.o = o
    def parse_macros(self):
        "parse_macros()"
        global out
        if not o.quiet: sys.stderr.write("macros:")
        for page in out.pages("macro"):
            if not o.quiet: sys.stderr.write(".")
            page.src.parse()
        if not o.quiet: sys.stderr.write(n)
    def macro_categories(self):
        print "macro_categories()"
        for page in out.sorted_pages("macro"):
            page.categories = IndexCategory(page.src)
    def gen_all_macro_html(self):
        for page in out.sorted_pages("macro"):
            page.htmlpage = HtmlMacroPage(page.src, page.categories)
    def gen_all_macro_xml(self):
        for page in out.sorted_pages("macro"):
            page.xmlpage = XmlMacroPage(page.src, self.o)
    def parse_and_gen(self):
        self.parse_macros()
        self.macro_categories()
        self.gen_all_macro_html()
        self.gen_all_macro_xml()

# ----------------------------------------------------
def help(arg):
    if o.cgi:
        help_cgi(arg)
    else:
        print arg
        help_args()
def help_args(arg = ""):
    print "ac_archive_macro_to_xml [html|xml|docbook] [file]..."
    print "  convert macro into html or docbook xml text"
    print "  being printed to stdout"
    print "options:"
    print "--package=name set package to be reported"
    print "--version=name set version to be reported"
    print "--release=name set release to be reported"
    print "--title=name set title for the cgi page"
    print "--quiet do not show progess messages"
    print "example:"
    print "  macro_to xml ax_echo_n.m4"
    print "note:"
    print " a REQUEST_METHOD environment variable will make"
    print " this script to be run as a www cgi program!"
    print "                               (C) Guido U. Draheim"
def help_cgi(arg):
    print "Status: 200 OK"
    print "Content-Type: text/html"
    print ""
    print '<html><head><title> Convert Autoconf Macro to Html or Docbook </title>'
    print '</head><body><h2> Convert Autoconf Macro to Html or Docbook </h2>'
    print '<p>',arg,"</p>"
    print '<FORM action="'+cgipath+'" method="POST">'
    print '<table><tr>'
    print '<td align="left">Macro Text:  <INPUT type="text" name="title"></td>'
    print '<td align="center"> &nbsp; &nbsp; &nbsp; &nbsp; '
    print 'HTML <INPUT type="radio" name="format" value="html" checked> or '
    print 'xml/manpage <INPUT type="radio" name="format" value="xml"></td>'
    print '<td align="right"><input type="submit" value="Convert!" /></td>'
    print '</tr><tr><td colspan="3">'
    print '<textarea cols="100" rows="100" name="text"></textarea>'
    print '</td></tr></table>'
    print '</FORM>'
    print "</body></html>"

def main(argv):
    o.version = "0.5.66"
    o.package = "ac-archive"
    o.topic = "macros"
    if "REQUEST_METHOD" in os.environ:
        o.cgi = os.environ["REQUEST_METHOD"]
    if len(argv) > 1:
        o.format = argv[0]
        argv = argv[1:]
        if o.format == "cgi": o.cgi = "!" ; o.format = "xml"
    if o.cgi:
        o.quiet = "!"
        scan_cgi(argv)
    else:
        scan_args(argv)
    if not o.format:
        help(""); return 0
    comp = ParseAndGen(o)
    comp.parse_and_gen()
    if o.format == "html":
        doctext = HtmlGenerator(o).with_style(style).gettext(out)
        if o.cgi:
            print "Status: 200 OK"
            print "Content-Type: text/html"
            print ""
        print doctext
    elif o.format == "xml" or o.format == "docbook":
        doc = DocbookDocument(o)
        for page in out.pages("macro"):
            doc.add(page)
        if o.cgi: doc.stylesheet = "sdocbook.css"
        doctext = doc.gettext()
        if o.cgi:
            print "Status: 200 OK"
            print "Content-Type: text/xml"
            print ""
        print doctext
    else:
        help("Unknown Format")
        return 1
    
if __name__ == "__main__":
    main(sys.argv[1:])
    
