#! /usr/bin/env python
# -*- coding: utf-8 -*- python -*-
# @creator (C) 2007 Guido U. Draheim
# @license http://creativecommons.org/licenses/by-nc-sa/2.0/de/
"""
Scan the arguments and build a proper webpage from the input.
"""

import re
import sys
import os
import os.path
import glob
# sys.path += [ "@site_packages@" ]
from ac_archive.os_ import *
from ac_archive.warn_ import *
from ac_archive.sorted_ import *
from ac_archive.options import *
from ac_archive.match import *
from ac_archive.subdirs import *
from ac_archive.navbar import *
from ac_archive.macrofile import *
from ac_archive.docusfile import *
from ac_archive.extrafile import *
from ac_archive.spellcheck import *
from ac_archive.indexgen import *
from ac_archive.htmlgen import *
from ac_archive.docbookdocument import *
from ac_archive.extras import *

# ---------------------------------------------------------- options / style
o = Options()

o.top_builddir = "."
o.topx = "macros"
n  = "\n"
br = "<br />\n"

style = AskStyleExtras()

class Page:
    def __init__(self, src):
        self.src = src
        self.htmlpage = None
        self.xmlpage = None
        self.categories = None
    def pagetype(self):
        if self.src:
            return self.src.pagetype()
        elif self.htmlpage:
            return self.htmlpage.pagetype()
        elif self.xmlpage:
            return self.htmlpage.pagetype()
        else:
            return None
    def xml_text(self):
        if not self.xmlpage: return None
        return self.xmlpage.xml_text()

class OutputFiles:
    def __init__(self):
        self.files = {}
    def add_src(self, name, page):
        self.files[name] = Page(page)
    def pages(self, pagetype):
        for page in self.files.values():
            if page.pagetype() == pagetype:
                yield page
    def sorted_pages(self, pagetype):
        for name in sorted(self.files.keys()):
            page = self.files[name]
            if page.pagetype() == pagetype:
                yield page

out = OutputFiles()

class SrcFiles:
    def __init__(self, outputfiles):
        self.outputfiles = outputfiles
        self.directories = []
    def add_directory(self, dirname):
        self.directories += [ dirname ]
    def scan_m4(self, filename, into = None):
        target = filename[:-len(".m4")]
        self.outputfiles.add_src(target, MacroFile(filename, into))

def scan_args(argv):
    files = SrcFiles(out)
    for arg in argv:
        try:
            if o.scan(arg): continue
            if arg.endswith(".m4"):  files.scan_m4(arg)
            if os.path.isdir(arg): files.add_directory(arg)
        except IOError, e:
            print arg,":", e
    return files.directories

class ParseAndGen:
    def __init__(self, o):
        self.o = o
    def parse_macros(self):
        "parse_macros()"
        global out
        if not o.quiet: sys.stderr.write("macros:")
        for page in out.pages("macro"):
            if not o.quiet: sys.stderr.write(".")
            page.src.parse()
        if not o.quiet: sys.stderr.write(n)
    def macro_categories(self):
        print "macro_categories()"
        for page in out.sorted_pages("macro"):
            page.categories = IndexCategory(page.src)
    def gen_all_macro_html(self):
        for page in out.sorted_pages("macro"):
            page.htmlpage = HtmlMacroPage(page.src, page.categories)
    def gen_all_macro_xml(self):
        for page in out.sorted_pages("macro"):
            page.xmlpage = XmlMacroPage(page.src, self.o)
    def parse_and_gen(self):
        self.parse_macros()
        self.macro_categories()
        self.gen_all_macro_html()
        self.gen_all_macro_xml()

# ---------------------------------------------------- 
def help():
    print "ac_archive_macro_to_html [file]..."
    print "options:"
    print "--package=name set package to be reported"
    print "--version=name set version to be reported"
    print "--release=name set release to be reported"
    print "--quiet do not show progess messages"
    print "--html=ext filename extension for html files"
    print "--output=directory for the html report files"
    print "                               (C) Guido U. Draheim"

def main(argv):
    o.version = "0.5.66"
    o.package = "ac-archive"
    o.topic = "macros"
    o.output = "."
    # print style.expand("<DT>foo</DT>")
    scan_args(argv)
    comp = ParseAndGen(o)
    comp.parse_and_gen()
    HtmlGenerator(o).with_style(style).generate_output(out)
    
if __name__ == "__main__":
    main(sys.argv[1:])
    
